REBOL [
	Title: "Functions to Register REBOL in Windows"
	Rights: "Copyright REBOL Technologies 1997-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

register: func [
	; Install program by setting registry values.
	maker [string!] ; REBOL
	name [string!]	; View
	version [tuple!]
	home [file!]	; Home directory
	sandbox [file!]	; Public data directory
	target [file!]	; program file
	/local key
][
	;-- Create uninstall link:
	key: join ms-uninstall reform [maker name]
	if not create-reg/hklm key [return false]
	set-reg/hklm key "DisplayName" to-string reduce [maker "/" name]
	set-reg/hklm key "UninstallString" rejoin [{"} to-local-file target {" -u}]

	;-- Create main registry key:
	key: reg-base maker name
	if not create-reg key [return false]
	set-reg key "Home" to-local-file home
	set-reg key "Version" form version
	set-reg key "Date" form now
	set-reg key "Sandbox" to-local-file sandbox
	set-reg key "Product" reform [name version]

	return true
]

unregister: func [
	; Uninstall main registry entry:
	maker [string!]
	name [string!]
][
	;-- Remove product key:
	delete-regs reg-base maker name

	;-- Find all products. Delete base key if no more products.
	keys: list-reg join "Software\" maker
	if all [keys empty? keys] [delete-regs join "Software\" maker]
	delete-regs/hklm join ms-uninstall reform [maker name]
	true
]

registered?: func [
	; If installed (registered) return version and home dir:
	maker [string!]
	name [string!]
	/local vers home data
][
	if winos? [ ; Windows
		return if all [
			vers: attempt [load get-reg reg-base maker name "Version"]
			tuple? vers
			home: attempt [dirize to-rebol-file get-reg reg-base maker name "Home"]
			data: attempt [dirize to-rebol-file get-reg reg-base maker name "Sandbox"]
		][reduce [vers home data]]
	]

	; Punt (for now) on other systems:
	vers: system/version
	home: default-prog-dir?
	data: default-data-dir?
	attempt [make-dir/deep data]
	reduce [vers home data]
]

;do %utils.r
;print "----------"
;probe registered? "REBOL" form system/product
;print unregister "REBOL" "View"
;probe registered? "REBOL" "View"
;print register "REBOL" "View" system/version %/f/rebol/view %/c/temp %/f/rebol/view/rebol.exe
;probe registered? "REBOL" "View"
