REBOL [
	Title: "REBOL Mezzanine Functions: XML"
	Rights: "Copyright REBOL Technologies 1997-2002"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

xml-language: make object! [
	verbose: off

	;-- Character Sets:
	joinset: func [cset chars] [insert copy cset chars]
	diffset: func [cset chars] [remove/part copy cset chars]
	error:   func [msg arg] [print [msg arg] halt]

	space:  charset [#"^(09)" #"^(0A)" #"^(0D)" #" "]
	char:   charset [#"^(09)" #"^(0A)" #"^(0D)" #" " - #"^(FF)"]
	letter: charset [#"A" - #"Z" #"a" - #"z"  #"^(C0)" - #"^(D6)" #"^(D8)" - #"^(F6)" #"^(F8)" -#"^(FF)" ]
	digit:  charset [#"0" - #"9"]

	alpha-num:  joinset letter "0123456789"  ; need to allow: digit
	name-first: joinset letter "_:"
	name-chars: joinset alpha-num ".-_:"
	data-chars: diffset char "<" ;"&<"
	qt1: "'"
	qt2: "^""
	data-chars-qt1: diffset data-chars qt1
	data-chars-qt2: diffset data-chars qt2

	name: [name-first any name-chars]
	sp:  [some space]
	sp?: [any space]

	;-- Tag Tree:
	parents: make block! 10


	;-- Tree Building Functions:
	new-node: func [name] [
		if verbose [print ["New tag:" name]]
		insert/only tail parents parent
		parent: add-kid copy reduce [name none none]
	]

	end-node: func [name] [
		while [name <> first parent] [
			if verbose [print ["unterminated tag:" first parent]]
			if empty? parents [error "End tag error:" name]
			pop-parent
		]
		pop-parent
	]

	pop-parent: func [] [
		parent: last parents
		remove back tail parents
	]

	add-kid: func [kid] [
		if none? third parent [parent/3: make block! 1]
		insert/only tail third parent kid
		kid
	]

	add-attr: func [name value] [
		if none? second parent [parent/2: make block! 2]
		insert insert tail second parent name value
	]

	check-version: func [version] [if verbose [print ["XML Version:" version]]]

	;-- XML Rules:
	document: [prolog sp? content to end]
	prolog: [sp? xml-decl? any [sp? doc-type-decls]]
	xml-decl?: ["<?xml" version-info thru "?>" | none]
	version-info: [sp "version" eq [qt1 version-num qt1 | qt2 version-num qt2]]
	version-num: [copy temp some name-chars (check-version temp)]
	doc-type-decls: [cmt | "<!" thru ">" | "<?" thru "?>"]

	element: [cmt | s-tag ["/>" (pop-parent) | #">" any content e-tag]]
	s-tag: [#"<" tag (node: new-node tag-name) any [sp attribute] sp?]
	e-tag: ["</" tag (end-node tag-name) sp? #">"]
	tag:   [copy tag-name name]
	content: [element | copy data some data-chars (add-kid data)]
	attribute: [copy attr-name name eq attr-value (add-attr attr-name attr-data)]
	eq: [sp? #"=" sp?]
	attr-value: [
		[qt1 copy attr-data any data-chars-qt1 qt1] |
		[qt2 copy attr-data any data-chars-qt2 qt2]
	]
	cmt: ["<!--" thru "-->"]

	parse-xml: func [str] [
;		trace on
		paroot: parent: copy reduce ['document none none]
		parse/case/all str document
		paroot
	]
]

parse-xml: func [
	"Parses XML code and returns a tree of blocks."
	code [string!] "XML code to parse"
][
	xml-language/parse-xml code
]

