REBOL [
	Title:  "REBOL Protocols: Set-Net"
	Author: ["Carl Sassenrath" "Sterling Newton" "Gabriele Santilli"]
	Rights: "Copyright REBOL Technologies 1997-2005"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

set-net: func [
	"Network setup.  All values after default are optional.  Words OK for server names."
	settings [block!] "[email-addr default-server pop-server proxy-server proxy-port-id proxy-type esmtp-user esmtp-pass]"
	/local actions item
][
	actions: [
		[system/user/email: item]
		[system/schemes/default/host: item]
		[system/schemes/pop/host: item]
		[system/schemes/default/proxy/host: item]
		[system/schemes/default/proxy/port-id: item]
		[system/schemes/default/proxy/type: item]
		[system/schemes/esmtp/user: item]
		[system/schemes/esmtp/pass: item]
	]
	forall settings [
		if (item: first settings) = 'none [item: none]
		if word? item [item: form item]
		do first actions
		if tail? actions: next actions [break]
	]
	if string? system/schemes/default/proxy/type [
		system/schemes/default/proxy/type: to-word system/schemes/default/proxy/type
	]
	if all [system/schemes/default/proxy/host none? system/schemes/default/proxy/type] [
		system/schemes/default/proxy/type: 'socks
	]
	if system/user/email [
		system/schemes/ftp/pass: system/user/email
	]
	; create functions to ask for proxy authentication if needed and not given
	all [
		none? get in system/schemes/default/proxy 'user
		system/schemes/default/proxy/user: func [] [
			system/schemes/default/proxy/user:
			ask "Enter proxy authentication username: "
		]
	]
	all [
		none? get in system/schemes/default/proxy 'pass
		system/schemes/default/proxy/pass: func [] [
			system/schemes/default/proxy/pass:
			ask/hide "Enter proxy authentication password: "
		]
	]
	if 'ask = pick head settings 7 [
		system/schemes/esmtp/user: func [] [
			system/schemes/esmtp/user:
			ask "Enter ESMTP authentication username: "
		]
		system/schemes/esmtp/pass: func [] [
			system/schemes/esmtp/pass:
			ask/hide "Enter ESMTP authentication password: "
		]
	]
	exit
]
