REBOL [
	Title: "REBOL/View Desktop - Icon Information"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

ii-object: ii-name: ii-type: ii-item: ii-info:
ii-size: ii-date: ii-path: ii-image:
	none

;-- Special styles used by desktop code:
info-styles: stylize [
	rt: text bold 80x22 middle right font-size 11
;	fl: field 250x22 font-size 11 para []
	ff: info  250x22 font-size 11 para [] snow
	fl: ff
	tgl: toggle 75x22 font-size 11 middle
	rtr: rotary 75x22 font-size 11 middle
]

;-- Icon Information Popup Layout: 
ii-lay: center-face layout [
	across origin 6
	backeffect base-effect
	styles info-styles
	space 2x4
	h2 "Icon Information:" return
	rt "Name:" ii-name: fl return
	rt "Type:" ii-type: ff 80
	rt "Size:" 46 ii-size: ff 120 return
	rt "Date:" ii-date: ff return
	rt "Path:" ii-path: ff 250x48 wrap return
	rt "Info:" ii-info: fl 250x80 wrap return

	space 8x4
	rt pad -6x6

	across
	btn-enter 75 "Edit" #"e" [
		hide-icon-info
		either any [file? ii-path/text url? ii-path/text][
			read-binary ii-path/text any [ii-object/stats true]
			editor path-thru ii-path/text
		][
			alert reform ["Cannot edit:" ii-path/text]
		]
	]
	btn 75 "Reload" #"r" [hide-icon-info read-binary ii-path/text none]
	btn-cancel 75 "Cancel" escape [hide-icon-info]
]

view-req: func [lay] [view/new/title lay "Request"]

show-icon-info: func [v /local file size] [
	; Show information about an icon (right click on icon):
	hide-icon-info
	unfocus
;	hide-folder-options
	ii-object: v
	ii-name/text: v/name
	ii-size/text:
	ii-date/text: "Not loaded"
	ii-path/text: v/item
	if word? v/item [ii-date/text: "internal"]
	if all [any [url? v/item file? v/item] find [file folder] v/type] [
		file: path-thru ii-path/text: full-path v/item
		if all [file? file exists? file][ ; file: full-path file] [
			;ii-path/text: file
			size: size? file
			tsize: form either size > 10240 [size / 1024][size]
			if find tsize "." [clear next next find tsize "."]
			ii-size/text: join tsize pick [" KB" " bytes"] size > 10240
			ii-date/text: modified? file
		]
	]
	ii-type/text: v/type
	ii-info/text: v/info
	ii-name/para/scroll: 0x0
	ii-size/para/scroll: 0x0
	view-req ii-lay
]

hide-icon-info: does [unview/only ii-lay]
